/*--------------------------------------------------------------------------------------+
|
|     $Source: MstnExamples/DgnEC/PickListExamples/NativeExample/PickListNativeExample.cpp $
|
|  $Copyright: (c) 2018 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|									|
|    $Logfile:   J:MstnExamples/DgnEC/PickListExamples/NativeExample/PickListNativeExample.cppv  $
|   $Workfile:   PickListNativeExample.cpp  $
|   	$Date:   2018/09/27    $
|									|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|									|
|   Include Files   							|
|									|
+----------------------------------------------------------------------*/
#include    <Mstn/MdlApi/MdlApi.h>
#include    <Mstn/MdlApi/mselems.h>
#include    <Mstn/isessionmgr.h>
#include    <Bentley/WString.h>
#include    <DgnPlatform/DgnECManager.h>
#include    <DgnPlatform/PickList.h>
#include    "PickListNativeExample.h"
#include    "PickListNativeExampleCmd.h"

USING_NAMESPACE_BENTLEY
USING_NAMESPACE_BENTLEY_DGNPLATFORM

using namespace Bentley::ECN;

/*---------------------------------------------------------------------------------**//**
* @LASolutions_method                                                LA Solutions Ltd
+---------------+---------------+---------------+---------------+---------------+------*/
void InstanceExample (WCharCP unused)
{
	//	Allocate a PickListLibrary: compiles OK, runs OK
	PickListLibraryP pLib { PickListLibrary::CreatePickListLibrary () };
	PickListLibrary::GetPickListLibFromDgn (ISessionMgr::GetActiveDgnFile (), *pLib);
	const size_t	nLists1 { pLib->GetPickListCount () };
	WString msg;
	msg.Sprintf (L"Dynamic PickListLibrary: nLists=%d", nLists1);
	mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Debug, msg.c_str (), msg.c_str (), DgnPlatform::OutputMessageAlert::Balloon);
	//	Create PickListLibrary on stack: compiles OK, fails at runtime
	PickListLibrary		lib;
	PickListLibrary::GetPickListLibFromDgn (ISessionMgr::GetActiveDgnFile (), lib);
	const size_t	nLists2 { lib.GetPickListCount () };
	msg.Sprintf (L"Stack-based PickListLibrary: nLists=%d", nLists2);
	mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Debug, msg.c_str (), msg.c_str (), DgnPlatform::OutputMessageAlert::Balloon);
}

/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                            Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DeleteAllPickLists(WCharCP unused)
    {
    //Get active DGN file.
    DgnFileP dgnFile = ISessionMgr::GetActiveDgnFile();

    PickListLibrary::DeletePickListLibFromDgn(dgnFile);
    }

/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                            Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void CreatePickListLibraryAndPickLists(WCharCP unused)
    {
    DgnFileP dgnFile = ISessionMgr::GetActiveDgnFile();

    PickListLibraryPtr pickListLib = PickListLibrary::CreatePickListLibrary();
    if(pickListLib == nullptr)
        {
        assert(pickListLib != nullptr);
        return;
        }
    PickListP pickList1 = pickListLib->AddPickList(L"Manufacturer", dgnFile);
    PickListP pickList2 = pickListLib->AddPickList(L"PowerType", dgnFile);
    PickListP pickList3 = pickListLib->AddPickList(L"Nation", dgnFile);

    if (pickList1 == nullptr ||
        pickList2 == nullptr ||
        pickList3 == nullptr
        )
        {
        assert(pickList1 != nullptr);
        assert(pickList2 != nullptr);
        assert(pickList3 != nullptr);
        return;
        }

    pickList1->AddValue(L"Ford");
    pickList2->AddValue(L"Gasoline");
    pickList2->AddValue(L"Electric");
    pickList3->AddValue(L"Germany");
    pickList3->AddValue(L"USA");

    PickListP manufacturerPickList = pickListLib->GetPickListByName(L"Manufacturer");
    if(nullptr == manufacturerPickList)
        {
        assert(manufacturerPickList != nullptr);
        return;
        }

    PickListP powerTypePickList = pickListLib->GetPickListById(2);
    if (nullptr == powerTypePickList)
        {
        assert(powerTypePickList != nullptr);
        return;
        }
    assert(powerTypePickList->GetName().CompareTo(L"PowerType") == 0);

    assert(pickListLib->RemovePickListByName(L"Nation"));
    PickListP pickList4 = pickListLib->AddPickList(L"Nation", dgnFile);
    if(nullptr == pickList4)
        {
        assert(pickList4 != nullptr);
        return;
        }
    assert(pickList4->GetId() == 4);
    assert(pickListLib->RemovePickListById(4));
    assert(pickListLib->GetPickListByName(L"Nation") == nullptr);
    assert(pickListLib->GetPickListCount() == 2);

    PickListP pickList5 = pickListLib->AddPickList(L"Nation", dgnFile);
    if (nullptr == pickList5)
        {
        assert(pickList5 != nullptr);
        return;
        }
    assert(pickList5->GetId() == 5);

    PickListLibrary::iterator iter = pickListLib->begin();
    for (; iter != pickListLib->end(); iter++)
        {
        UInt32 id = ((PickListP)(&(*iter)))->GetId();
        if (1 == id)
            assert(pickListLib->GetPickListById(id)->GetName().CompareTo(L"Manufacturer") == 0);
        else if (2 == id)
            assert(pickListLib->GetPickListById(id)->GetName().CompareTo(L"PowerType") == 0);
        else if (5 == id)
            assert(pickListLib->GetPickListById(id)->GetName().CompareTo(L"Nation") == 0);
        else
            {
            assert(0);
            return;
            }
        }

    PickListLibrary::SavePickListLibToDgn(dgnFile, *pickListLib, false);
    }

/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                            Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
extern "C" DLLEXPORT void MdlMain
(
    int         argc,
    WCharCP     argv[]
)
    {
    MdlCommandNumber commandNumbers[] =
        {
            { (CmdHandler)CreatePickListLibraryAndPickLists, CMD_PICKLISTNATIVEEXAMPLE_CREATE },
            { (CmdHandler)DeleteAllPickLists, CMD_PICKLISTNATIVEEXAMPLE_DELETEALL },
            { (CmdHandler)InstanceExample, CMD_PICKLISTNATIVEEXAMPLE_INSTANCE },
            0
        };
    mdlSystem_registerCommandNumbers(commandNumbers);

    RscFileHandle   rfHandle;

    /* open our application file for access to our dialogs */
    mdlResource_openFile(&rfHandle, NULL, RSC_READ);

    if (mdlParse_loadCommandTable(NULL) != NULL)
        mdlOutput_rscPrintf(MSG_MESSAGE, NULL, MESSAGELISTID_PickListNativeExampleMessages, MSG_commandsLoaded);
    }